/** \file easytx52_1010.c
* \brief TX-1010
* \brief This program transmits 1010... data via AX501
*/

//Copyright 2015 ON Semiconductor.  All rights reserved.  This software and/or related material is licensed by ON Semiconductor under limited terms and conditions.  The 
//terms and conditions pertaining to the software and/or related material are available at http://www.onsemi.com/site/pdf/ONSEMI_T&C.pdf (“ON Semiconductor Standard Terms 
//and Conditions of Sale, Section 8 Software”).  Do not use this software and/or related material unless you have carefully read and you agree to the limited terms and 
//conditions. By using this software and/or related material, you agree to the limited terms and conditions.


#include <ax8052f131.h>
#include <libmflcd.h>
#include <libmfradio.h>

#include "global.h"




uint8_t __data   pllrange;   	   	// Synthesizer range
uint32_t __data bytecnt;            // tx byte counter




/************************************************************************/
/************************************************************************/
/* Initialization Routines                                              */
/************************************************************************/
/************************************************************************/



uint8_t _sdcc_external_startup(void)
{
    PORTA = 0xFF;
    PORTB = 0xFF;
    PORTC = 0xFF;
    PORTR = 0xEA;

    DIRA = 0x00;
    DIRB = 0x00;
    DIRC = 0x00;
    DIRR = 0x15;

//    SPCLKSRC = 0xD8;
//    SPMODE = 0x01;
//    SPSHREG;

    RADIOMUX = 0x4F;
    RADIOACC = 0x00;

    EIE = 0x00;
    E2IE = 0x00;
    IE = 0x10;  // Enable radio IRQ

    return 0;
}


void stop_with_error(const __code char *str)
{
    lcd_setpos(0);
    lcd_waitshort();
    lcd_writestr(str);
    while(1)
        enter_sleep();
}

// ------------------------------------------------------------------------
// ------------------------------------------------------------------------


void init_ax5031(void)
{

    // register settings are calculated by the AXEVK software
    // The code snippets window was pasted into this routine
    // A few registers were added by hand, see comments.

    AX5031_PWRMODE = 0x00;   	   // mode OFF
    AX5031_PINCFG1 = 0xF0;
    AX5031_PINCFG2 = 0xC0; 			// required for module 7.3

    // ---------------------------------------------------------
    //
    // copy from EVK code snippets

#include "ax5031-fsk-100k-1010.h"

    // ---------------------------------------------------------



} // end init_ax5031



uint8_t pllrange_ax5031(uint8_t range)    	   	// range the AX501 Synthesizer
{
    uint8_t r;
    uint8_t rcount=0;

    AX5031_PWRMODE = 0x00;
    AX5031_PWRMODE = 0x0C;   	// set mode synth TX

    r=range;
    AX5031_PLLRANGING = 0x10 | (range & 0x0f);  // start value typical 0x08

    do
    {
        delay(2000);
        r=AX5031_PLLRANGING;
        rcount++;
        if (rcount>10) stop_with_error("PLL range\nERROR");
    }
    while ((r&0x10)!=0);

    r=r&0x0F;
    return r;
}


void set_ax5031_idle(void) __reentrant   	// put the program and the AX5031 into idle mode
{
    AX5031_IRQMASK = 0x00;      // disable AX501 interrupts
    EA = 0;    	   	   	        // disable 8051 IRQ
    AX5031_PWRMODE = 0x05;      // REF=ON, OSC=ON

}  // end set_ax5031_idle


// ------------------------------------------------------------------------------------------

void PLL_lock(void)
{
    uint8_t u8=0;


    while (u8 == 0)
    {
        u8 = AX5031_PLLRANGING;
        u8 &= 0x40;
    }
}

void set_ax5031_TX(void) __reentrant   	   // initializes the TX process
{
    uint8_t i;

    AX5031_PWRMODE = 0x05;  // REF=ON, OSC=ON
    delay(20000);
    AX5031_PWRMODE = 0x0C;  // REF=ON, OSC=ON, Synthesizer running
    PLL_lock();
    AX5031_TXPWR = 0x00;
    AX5031_PWRMODE = 0x0D;  // REF=ON, OSC=ON, Synthesizer and tx running

    for (i = 1; i < 16; ++i)
        AX5031_TXPWR = i;

    AX5031_FIFODATA = 0x55;
    AX5031_FIFODATA = 0x55;
    AX5031_FIFODATA = 0x55;
    AX5031_FIFODATA = 0x55;

    AX5031_IRQMASK = 0x02;    // interrupt when fifo not full
    EA = 1;  	   	   	   // enable 8051 IRQ

}  // end set_ax5031_TX


// ------------------------------------------------------------------------------------------


void set_clk_ax5031(uint8_t clock)   	   // switch sysclk on for testing
{

    AX5031_PWRMODE = 0x05;     // ref and osc ON
    delay(10000);   	   	   	   	// wait for oscillator to be settled
    if (clock==OFF)
        AX5031_PINCFG1 =   0xF0;
    if (clock==16)
        AX5031_PINCFG1 =   0xF4;

}



// ----------------------------------------------------------------------
//

void lcd_clear(uint8_t pos, uint8_t num) {
    __xdata char clr[] = "                ";
    lcd_setpos(pos);
    lcd_waitshort();
//    if(pos & 0x40)
//        pos &= ~0x40;  // take out the "second line" bit

    if(num<=16) {
        clr[num] = 0x00;
        lcd_writestr(clr);
        clr[num] = ' ';
    }
}

void sig_and_abs16 (int16_t val, char *sgn, uint16_t *abs) {
    if(val & 0x8000) {
        *sgn=-1;
        *abs = -val;
    }
    else{
        *sgn = 1;
        *abs = val;
    }
}


// ----------------------------------------------------------------------
//

uint8_t u8;

void main (void)
{
#ifndef SDCC
    _sdcc_external_startup();
#endif

    flash_apply_calibration();

    lcd_init();
    lcd_setpos(0);
    lcd_waitshort();

    ax5031_comminit();

    if (ax5031_reset())
        stop_with_error("No AX5031 found");

    init_ax5031();

    lcd_writestr("AX5031 TX 1010");

    pllrange = pllrange_ax5031(0x08);

    set_ax5031_idle();
    set_clk_ax5031(OFF); // used to measure sysclk frequency with counter for frequency calibration

    bytecnt=0;

 //   for (u8=50; u8>0; u8++)
 //       delay(10000);

    set_ax5031_TX();

    lcd_setpos(0x47);
    lcd_waitshort();
    lcd_writestr("kBytes");

    while(1)
    {
        if (bytecnt % 1024)
        {
            lcd_setpos(0x45);
            lcd_waitshort();
            lcd_writeu32(bytecnt>>10, 6, 0x45);
        }
    }

}

// ------------------------------------------------------------------------
// ------------------------------------------------------------------------

// ISR functions


void AX_IRQ(void) __interrupt(4)
{

    AX5031_FIFODATA = 0x55;
    bytecnt++;

} //end AX_IRQ


